#Autogenerated by ReportLab guiedit do not edit
from reportlab.lib.corp import RL_CorpLogo
from reportlab.graphics.shapes import Drawing, _DrawingEditorMixin
from reportlab.lib.colors import white, ReportLabBlue

class RL_LOGO(_DrawingEditorMixin,Drawing):
    def __init__(self,width=400,height=200,*args,**kw):
        Drawing.__init__(self,width,height,*args,**kw)
        self._add(self,RL_CorpLogo(),name='logo',validate=None,desc=None)
        self.logo.fillColor   = ReportLabBlue
        self.logo.background  = white
        self.logo.shadow      = .7
        self.height     = 86
        self.width      = 130
    def getContents(self):
        self.logo.width=self.width
        self.logo.height=self.height
        return Drawing.getContents(self)

if __name__=="__main__": #NORUNTESTS
    RL_LOGO().save(formats=['pdf'],outDir='.',fnRoot=None)
